@@ -41,9 +41,7 @@
 #include "champlain-defines.h"
 #include "champlain-marshal.h"
 #include "champlain-private.h"
-#include "champlain-map.h"
 #include "champlain-tile.h"
-#include "champlain-zoom-level.h"
 
 #include <clutter/clutter.h>
 #include <glib.h>
@@ -110,15 +108,14 @@ struct _ChamplainMarkerPrivate
 
 G_DEFINE_TYPE (ChamplainMarker, champlain_marker, CHAMPLAIN_TYPE_BASE_MARKER);
 
-#define CHAMPLAIN_MARKER_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), CHAMPLAIN_TYPE_MARKER, ChamplainMarkerPrivate))
+#define GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), CHAMPLAIN_TYPE_MARKER, ChamplainMarkerPrivate))
 
 static void draw_marker (ChamplainMarker *marker);
 
 /**
- * champlain_marker_set_hightlight_color:
+ * champlain_marker_set_highlight_color:
  * @color: a #ClutterColor
  *
- *
  * Changes the highlight color, this is to ensure a better integration with
  * the desktop, this is automatically done by GtkChamplainEmbed.
  *
@@ -134,7 +131,9 @@ champlain_marker_set_highlight_color (ClutterColor *color)
 }
 
 /**
- * champlain_marker_get_hightlight_color:
+ * champlain_marker_get_highlight_color:
+ *
+ * Gets the highlight color.
  *
  * Returns: the highlight color. Should not be freed.
  *
@@ -147,10 +146,9 @@ champlain_marker_get_highlight_color ()
 }
 
 /**
- * champlain_marker_set_hightlight_text_color:
+ * champlain_marker_set_highlight_text_color:
  * @color: a #ClutterColor
  *
- *
  * Changes the highlight text color, this is to ensure a better integration with
  * the desktop, this is automatically done by GtkChamplainEmbed.
  *
@@ -166,7 +164,9 @@ champlain_marker_set_highlight_text_color (ClutterColor *color)
 }
 
 /**
- * champlain_marker_get_hightlight_text_color:
+ * champlain_marker_get_highlight_text_color:
+ *
+ * Gets the highlight text color.
  *
  * Returns: the highlight text color. Should not be freed.
  *
@@ -180,12 +180,11 @@ champlain_marker_get_highlight_text_color ()
 
 static void
 champlain_marker_get_property (GObject *object,
-                               guint prop_id,
-                               GValue *value,
-                               GParamSpec *pspec)
+    guint prop_id,
+    GValue *value,
+    GParamSpec *pspec)
 {
-    ChamplainMarker *marker = CHAMPLAIN_MARKER (object);
-    ChamplainMarkerPrivate *priv = marker->priv;
+    ChamplainMarkerPrivate *priv = CHAMPLAIN_MARKER (object)->priv;
 
     switch (prop_id)
       {
@@ -232,12 +231,11 @@ champlain_marker_get_property (GObject *object,
 
 static void
 champlain_marker_set_property (GObject *object,
-                               guint prop_id,
-                               const GValue *value,
-                               GParamSpec *pspec)
+    guint prop_id,
+    const GValue *value,
+    GParamSpec *pspec)
 {
     ChamplainMarker *marker = CHAMPLAIN_MARKER (object);
-    //ChamplainMarkerPrivate *priv = marker->priv;
 
     switch (prop_id)
     {
@@ -282,23 +280,72 @@ champlain_marker_set_property (GObject *object,
     }
 }
 
+static void
+champlain_marker_dispose (GObject *object)
+{
+  ChamplainMarkerPrivate *priv = CHAMPLAIN_MARKER (object)->priv;
+
+  if (priv->background)
+    {
+      g_object_unref (priv->background);
+      priv->background = NULL;
+    }
+
+  if (priv->shadow)
+    {
+      g_object_unref (priv->shadow);
+      priv->shadow = NULL;
+    }
+
+  if (priv->text_actor)
+    {
+      g_object_unref (priv->text_actor);
+      priv->text_actor = NULL;
+    }
+
+  if (priv->image)
+    {
+      g_object_unref (priv->image);
+      priv->image = NULL;
+    }
+
+  if (priv->attributes)
+    {
+      pango_attr_list_unref (priv->attributes);
+      priv->attributes = NULL;
+    }
+
+  G_OBJECT_CLASS (champlain_marker_parent_class)->dispose (object);
+}
+
 static void
 champlain_marker_finalize (GObject *object)
 {
-  ChamplainMarker *marker = CHAMPLAIN_MARKER (object);
-  ChamplainMarkerPrivate *priv = marker->priv;
+  ChamplainMarkerPrivate *priv = CHAMPLAIN_MARKER (object)->priv;
 
-  if (priv->text != NULL)
-    g_free (priv->text);
-  priv->text = NULL;
+  if (priv->text)
+    {
+      g_free (priv->text);
+      priv->text = NULL;
+    }
 
-  if (priv->image != NULL)
-    g_object_unref (priv->image);
-  priv->image = NULL;
+  if (priv->font_name)
+    {
+      g_free (priv->font_name);
+      priv->font_name = NULL;
+    }
 
-  if (priv->background != NULL)
-    g_object_unref (priv->background);
-  priv->background = NULL;
+  if (priv->color)
+    {
+      clutter_color_free (priv->color);
+      priv->color = NULL;
+    }
+
+  if (priv->text_color)
+    {
+      clutter_color_free (priv->text_color);
+      priv->text_color = NULL;
+    }
 
   if (priv->redraw_id)
     {
@@ -316,6 +363,7 @@ champlain_marker_class_init (ChamplainMarkerClass *markerClass)
 
   GObjectClass *object_class = G_OBJECT_CLASS (markerClass);
   object_class->finalize = champlain_marker_finalize;
+  object_class->dispose = champlain_marker_dispose;
   object_class->get_property = champlain_marker_get_property;
   object_class->set_property = champlain_marker_set_property;
 
@@ -551,7 +599,7 @@ draw_background (ChamplainMarker *marker,
     gint point)
 {
   ChamplainMarkerPrivate *priv = marker->priv;
-  ChamplainBaseMarkerPrivate *base_priv = CHAMPLAIN_BASE_MARKER (marker)->priv;
+  ChamplainBaseMarker *base_marker = CHAMPLAIN_BASE_MARKER (marker);
   ClutterActor *bg = NULL;
   ClutterColor *color;
   ClutterColor darker_color;
@@ -561,7 +609,7 @@ draw_background (ChamplainMarker *marker,
   cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (bg));
 
   /* If selected, add the selection color to the marker's color */
-  if (base_priv->highlighted)
+  if (champlain_base_marker_get_highlighted (base_marker))
     color = &SELECTED_COLOR;
   else
     color = priv->color;
@@ -603,7 +651,7 @@ static void
 draw_marker (ChamplainMarker *marker)
 {
   ChamplainMarkerPrivate *priv = marker->priv;
-  ChamplainBaseMarkerPrivate *base_priv = CHAMPLAIN_BASE_MARKER (marker)->priv;
+  ChamplainBaseMarker *base_marker = CHAMPLAIN_BASE_MARKER (marker);
   guint height = 0, point = 0;
   guint total_width = 0, total_height = 0;
 
@@ -652,7 +700,7 @@ draw_marker (ChamplainMarker *marker)
         total_height = height;
 
       clutter_text_set_color (CLUTTER_TEXT (priv->text_actor),
-          (base_priv->highlighted ? &SELECTED_TEXT_COLOR : priv->text_color));
+          (champlain_base_marker_get_highlighted (base_marker) ? &SELECTED_TEXT_COLOR : priv->text_color));
       if (clutter_actor_get_parent (priv->text_actor) == NULL)
         clutter_container_add_actor (CLUTTER_CONTAINER (marker), priv->text_actor);
     }
@@ -665,7 +713,7 @@ draw_marker (ChamplainMarker *marker)
 
   point = (total_height + 2 * PADDING) / 4.0;
 
-  if (priv->draw_background == TRUE)
+  if (priv->draw_background)
     {
       draw_shadow (marker, total_width, total_height, point);
       draw_background (marker, total_width, total_height, point);
@@ -692,7 +740,7 @@ draw_marker (ChamplainMarker *marker)
   if (priv->image != NULL && priv->background != NULL)
     clutter_actor_raise (priv->image, priv->background);
 
-  if (priv->draw_background == TRUE)
+  if (priv->draw_background)
   {
     if (priv->alignment == PANGO_ALIGN_RIGHT)
       clutter_actor_set_anchor_point (CLUTTER_ACTOR (marker), total_width, total_height + point);
@@ -718,7 +766,8 @@ redraw_on_idle (gpointer gobject)
   return FALSE;
 }
 
-/** champlain_marker_queue_redraw:
+/**
+ * champlain_marker_queue_redraw:
  * @marker: a #ChamplainMarker
  *
  * Queue a redraw of the marker as soon as possible. This function should not
@@ -731,14 +780,22 @@ redraw_on_idle (gpointer gobject)
 void
 champlain_marker_queue_redraw (ChamplainMarker *marker)
 {
-  if (!marker->priv->redraw_id)
-    marker->priv->redraw_id = g_idle_add (redraw_on_idle, marker);
+  ChamplainMarkerPrivate *priv = marker->priv;
+
+  if (!priv->redraw_id)
+    {
+      priv->redraw_id =
+          g_idle_add_full (G_PRIORITY_DEFAULT,
+                           (GSourceFunc)redraw_on_idle,
+                           g_object_ref (marker),
+                           (GDestroyNotify)g_object_unref);
+    }
 }
 
 static void
 notify_highlighted (GObject *gobject,
-    GParamSpec *pspec,
-    gpointer user_data)
+    G_GNUC_UNUSED GParamSpec *pspec,
+    G_GNUC_UNUSED gpointer user_data)
 {
   champlain_marker_queue_redraw (CHAMPLAIN_MARKER (gobject));
 }
@@ -746,7 +803,8 @@ notify_highlighted (GObject *gobject,
 static void
 champlain_marker_init (ChamplainMarker *marker)
 {
-  ChamplainMarkerPrivate *priv = CHAMPLAIN_MARKER_GET_PRIVATE (marker) ;
+  ChamplainMarkerPrivate *priv = GET_PRIVATE (marker);
+
   marker->priv = priv;
 
   priv->text = NULL;
@@ -771,6 +829,8 @@ champlain_marker_init (ChamplainMarker *marker)
 /**
  * champlain_marker_new:
  *
+ * Creates a new instance of #ChamplainMarker.
+ *
  * Returns: a new #ChamplainMarker ready to be used as a #ClutterActor.
  *
  * Since: 0.2
@@ -788,6 +848,8 @@ champlain_marker_new (void)
  * @text_color: a #ClutterColor, the color of the text, can be NULL
  * @marker_color: a #ClutterColor, the color of the marker, can be NULL
  *
+ * Creates a new instance of #ChamplainMarker with text value.
+ *
  * Returns: a new #ChamplainMarker with a drawn marker containing the given text.
  *
  * Since: 0.2
@@ -818,6 +880,8 @@ champlain_marker_new_with_text (const gchar *text,
  * champlain_marker_new_with_image:
  * @actor: The actor of the image.
  *
+ * Creates a new instance of #ChamplainMarker with image.
+ *
  * Returns: a new #ChamplainMarker with a drawn marker containing the given
  * image.
  *
@@ -840,6 +904,8 @@ champlain_marker_new_with_image (ClutterActor *actor)
  * @filename: The filename of the image.
  * @error: Return location for an error.
  *
+ * Creates a new instance of #ChamplainMarker with image loaded from file.
+ *
  * Returns: a new #ChamplainMarker with a drawn marker containing the given
  * image.
  *
@@ -868,6 +934,8 @@ champlain_marker_new_from_file (const gchar *filename,
  * @text: The text
  * @actor: The image
  *
+ * Creates a new instance of #ChamplainMarker consisting of a custom #ClutterActor.
+ *
  * Returns: a new #ChamplainMarker with a drawn marker containing the given
  * image.
  *
@@ -929,7 +997,7 @@ champlain_marker_set_image (ChamplainMarker *marker,
   ChamplainMarkerPrivate *priv = marker->priv;
 
   if (priv->image != NULL)
-    g_object_unref (priv->image);
+    clutter_actor_destroy (priv->image);
 
   if (image != NULL)
     {
@@ -958,9 +1026,7 @@ champlain_marker_set_use_markup (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->use_markup = markup;
+  marker->priv->use_markup = markup;
   g_object_notify (G_OBJECT (marker), "use-markup");
   champlain_marker_queue_redraw (marker);
 }
@@ -980,9 +1046,7 @@ champlain_marker_set_alignment (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->alignment = alignment;
+  marker->priv->alignment = alignment;
   g_object_notify (G_OBJECT (marker), "alignment");
   champlain_marker_queue_redraw (marker);
 }
@@ -1009,7 +1073,7 @@ champlain_marker_set_color (ChamplainMarker *marker,
     clutter_color_free (priv->color);
 
   if (color == NULL)
-     color = &DEFAULT_COLOR;
+    color = &DEFAULT_COLOR;
 
   priv->color = clutter_color_copy (color);
   g_object_notify (G_OBJECT (marker), "color");
@@ -1038,7 +1102,7 @@ champlain_marker_set_text_color (ChamplainMarker *marker,
     clutter_color_free (priv->text_color);
 
   if (color == NULL)
-     color = &DEFAULT_TEXT_COLOR;
+    color = &DEFAULT_TEXT_COLOR;
 
   priv->text_color = clutter_color_copy (color);
   g_object_notify (G_OBJECT (marker), "text-color");
@@ -1089,9 +1153,7 @@ champlain_marker_set_wrap (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->wrap = wrap;
+  marker->priv->wrap = wrap;
   g_object_notify (G_OBJECT (marker), "wrap");
   champlain_marker_queue_redraw (marker);
 }
@@ -1111,9 +1173,7 @@ champlain_marker_set_wrap_mode (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->wrap_mode = wrap_mode;
+  marker->priv->wrap_mode = wrap_mode;
   g_object_notify (G_OBJECT (marker), "wrap");
   champlain_marker_queue_redraw (marker);
 }
@@ -1162,9 +1222,7 @@ champlain_marker_set_ellipsize (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->ellipsize = ellipsize;
+  marker->priv->ellipsize = ellipsize;
   g_object_notify (G_OBJECT (marker), "ellipsize");
   champlain_marker_queue_redraw (marker);
 }
@@ -1183,9 +1241,7 @@ champlain_marker_set_single_line_mode (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->single_line_mode = mode;
+  marker->priv->single_line_mode = mode;
 
   g_object_notify (G_OBJECT (marker), "single-line-mode");
   champlain_marker_queue_redraw (marker);
@@ -1206,9 +1262,7 @@ champlain_marker_set_draw_background (ChamplainMarker *marker,
 {
   g_return_if_fail (CHAMPLAIN_IS_MARKER (marker));
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  priv->draw_background = background;
+  marker->priv->draw_background = background;
   g_object_notify (G_OBJECT (marker), "draw-background");
   champlain_marker_queue_redraw (marker);
 }
@@ -1217,6 +1271,8 @@ champlain_marker_set_draw_background (ChamplainMarker *marker,
  * champlain_marker_get_image:
  * @marker: The marker
  *
+ * Get the marker's image.
+ *
  * Returns: the marker's image.
  *
  * Since: 0.4
@@ -1226,15 +1282,15 @@ champlain_marker_get_image (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), NULL);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->image;
+  return marker->priv->image;
 }
 
 /**
  * champlain_marker_get_use_markup:
  * @marker: The marker
  *
+ * Check whether the marker uses markup.
+ *
  * Returns: if the marker's text contains markup.
  *
  * Since: 0.4
@@ -1244,15 +1300,15 @@ champlain_marker_get_use_markup (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->use_markup;
+  return marker->priv->use_markup;
 }
 
 /**
  * champlain_marker_get_text:
  * @marker: The marker
  *
+ * Get the marker's text.
+ *
  * Returns: the marker's text.
  *
  * Since: 0.4
@@ -1262,15 +1318,15 @@ champlain_marker_get_text (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->text;
+  return marker->priv->text;
 }
 
 /**
  * champlain_marker_get_alignment:
  * @marker: The marker
  *
+ * Get the marker's text alignment.
+ *
  * Returns: the marker's text alignment.
  *
  * Since: 0.4
@@ -1280,15 +1336,15 @@ champlain_marker_get_alignment (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->alignment;
+  return marker->priv->alignment;
 }
 
 /**
  * champlain_marker_get_color:
  * @marker: The marker
  *
+ * Gets the marker's color.
+ *
  * Returns: the marker's color.
  *
  * Since: 0.4
@@ -1298,15 +1354,15 @@ champlain_marker_get_color (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), NULL);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->color;
+  return marker->priv->color;
 }
 
 /**
  * champlain_marker_get_text_color:
  * @marker: The marker
  *
+ * Gets the marker's text color.
+ *
  * Returns: the marker's text color.
  *
  * Since: 0.4
@@ -1316,15 +1372,15 @@ champlain_marker_get_text_color (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), NULL);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->text_color;
+  return marker->priv->text_color;
 }
 
 /**
  * champlain_marker_get_font_name:
  * @marker: The marker
  *
+ * Gets the marker's font name.
+ *
  * Returns: the marker's font name.
  *
  * Since: 0.4
@@ -1334,15 +1390,15 @@ champlain_marker_get_font_name (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->font_name;
+  return marker->priv->font_name;
 }
 
 /**
  * champlain_marker_get_wrap:
  * @marker: The marker
  *
+ * Check whether the marker text wraps.
+ *
  * Returns: if the marker's text wraps.
  *
  * Since: 0.4
@@ -1352,15 +1408,15 @@ champlain_marker_get_wrap (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->wrap;
+  return marker->priv->wrap;
 }
 
 /**
  * champlain_marker_get_wrap_mode:
  * @marker: The marker
  *
+ * Get the marker's text wrap mode.
+ *
  * Returns: the marker's text wrap mode.
  *
  * Since: 0.4
@@ -1370,15 +1426,15 @@ champlain_marker_get_wrap_mode (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->wrap_mode;
+  return marker->priv->wrap_mode;
 }
 
 /**
  * champlain_marker_get_ellipsize:
  * @marker: The marker
  *
+ * Get the marker's text ellipsize mode.
+ *
  * Returns: the marker's text ellipsize mode.
  *
  * Since: 0.4
@@ -1388,16 +1444,16 @@ champlain_marker_get_ellipsize (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->ellipsize;
+  return marker->priv->ellipsize;
 }
 
 /**
  * champlain_marker_get_single_line_mode:
  * @marker: The marker
  *
- * Returns: the marker's text single side mode.
+ * Checks the marker's single line mode.
+ *
+ * Returns: the marker's text single line mode.
  *
  * Since: 0.4
  */
@@ -1406,15 +1462,15 @@ champlain_marker_get_single_line_mode (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->single_line_mode;
+  return marker->priv->single_line_mode;
 }
 
 /**
  * champlain_marker_get_draw_background:
  * @marker: The marker
  *
+ * Checks whether the marker has a background.
+ *
  * Returns: if the marker's has a background.
  *
  * Since: 0.4
@@ -1424,8 +1480,6 @@ champlain_marker_get_draw_background (ChamplainMarker *marker)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_MARKER (marker), FALSE);
 
-  ChamplainMarkerPrivate *priv = marker->priv;
-
-  return priv->draw_background;
+  return marker->priv->draw_background;
 }
 
