@@ -22,7 +22,7 @@
 #define PADDING 10
 
 static gboolean
-map_view_button_release_cb (ClutterActor *actor,
+map_view_button_release_cb (G_GNUC_UNUSED ClutterActor *actor,
     ClutterButtonEvent *event,
     ChamplainView * view)
 {
@@ -39,8 +39,8 @@ map_view_button_release_cb (ClutterActor *actor,
 }
 
 static gboolean
-zoom_in (ClutterActor *actor,
-    ClutterButtonEvent *event,
+zoom_in (G_GNUC_UNUSED ClutterActor *actor,
+    G_GNUC_UNUSED ClutterButtonEvent *event,
     ChamplainView * view)
 {
   champlain_view_zoom_in (view);
@@ -48,8 +48,8 @@ zoom_in (ClutterActor *actor,
 }
 
 static gboolean
-zoom_out (ClutterActor *actor,
-    ClutterButtonEvent *event,
+zoom_out (G_GNUC_UNUSED ClutterActor *actor,
+    G_GNUC_UNUSED ClutterButtonEvent *event,
     ChamplainView * view)
 {
   champlain_view_zoom_out (view);
@@ -87,7 +87,7 @@ main (int argc,
 {
   ClutterActor* actor, *stage, *buttons, *button;
   ChamplainLayer *layer;
-  gfloat width, total_width = 0;;
+  gfloat width, total_width = 0;
 
   g_thread_init (NULL);
   clutter_init (&argc, &argv);
@@ -118,7 +118,6 @@ main (int argc,
   clutter_actor_set_reactive (button, TRUE);
   clutter_actor_set_position (button, total_width, 0);
   clutter_actor_get_size (button, &width, NULL);
-  total_width += width + PADDING;
   g_signal_connect (button, "button-release-event",
       G_CALLBACK (zoom_out),
       actor);
@@ -143,5 +142,6 @@ main (int argc,
   clutter_actor_show (stage);
   clutter_main ();
 
+  clutter_actor_destroy (actor);
   return 0;
 }
